import { Sensor, Keyboard, Pointer, Scroll, KeyboardModifiers } from './Sensor';
import { AssociatedDataPayload, AssociatedDataImportance } from './AssociatedData';
export interface AbstractSensors {
    readonly focus: Readonly<Sensor>;
    readonly hover: Readonly<Sensor>;
    readonly keyboard: Readonly<Keyboard>;
    readonly pointer: Readonly<Pointer>;
    readonly scroll: Readonly<Scroll>;
}
export declare class Sensors implements AbstractSensors {
    readonly focus: Sensor;
    readonly hover: Sensor;
    readonly keyboard: Keyboard;
    readonly pointer: Pointer;
    readonly scroll: Scroll;
    resetFocus(): void;
    protected trackFocus(focus: unknown[], force: boolean): void;
    protected doFocusIn(focus: unknown[]): void;
    protected doFocusOut(focus: unknown[]): void;
    protected doPointerOver(associatedDataList: unknown[], hoverAssociatedDataList: unknown[], clientX: number, clientY: number): void;
    protected doPointerLeave(associatedDataList: unknown[], hoverAssociatedDataList: unknown[], clientX: number, clientY: number): void;
    protected doPointerMove(associatedDataList: unknown[], pointerId: number, clientX: number, clientY: number): void;
    protected doPointerDown(associatedDataList: unknown[], focus: unknown[], pointerId: number, buttons: number, clientX: number, clientY: number): void;
    protected doPointerUp(associatedDataList: unknown[], focus: unknown[], pointerId: number, buttons: number, clientX: number, clientY: number): void;
    protected doPointerCancel(associatedDataList: unknown[], focus: unknown[], pointerId: number, buttons: number, clientX: number, clientY: number): void;
    protected doClick(associatedDataList: unknown[], focus: unknown[], buttons: number, clientX: number, clientY: number): void;
    protected doDblClick(associatedDataList: unknown[], focus: unknown[], buttons: number, clientX: number, clientY: number): void;
    protected doTouchStart(associatedDataList: unknown[], focus: unknown[]): void;
    protected doTouchEnd(associatedDataList: unknown[]): void;
    protected doWheel(associatedDataList: unknown[], focus: unknown[], deltaX: number, deltaY: number, clientX: number, clientY: number): void;
    protected doKeyDown(associatedDataList: unknown[], key: string): void;
    protected doKeyUp(associatedDataList: unknown[], key: string): void;
    protected setPointerCapture(pointerId: number): boolean;
    protected releasePointerCapture(pointerId: number): boolean;
    protected static rememberPointer(p: Pointer, clientX: number, clientY: number): void;
    private static getKeyAsModifierIfAny;
    static extractModifierKeys(e: PointerEvent | KeyboardEvent | WheelEvent): KeyboardModifiers;
}
export declare function grabAssociatedDataList<T = unknown>(path: any[], sym: symbol, payloadKey: keyof AssociatedDataPayload, importanceKey: keyof AssociatedDataImportance, existing: Array<T>): T[];
export declare function switchAssociatedDataList(existing: unknown[], updated: unknown[]): unknown[];
