"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.switchAssociatedDataList = exports.grabAssociatedDataList = exports.Sensors = void 0;
const reactronic_1 = require("reactronic");
const Sensor_1 = require("./Sensor");
class Sensors {
    constructor() {
        this.focus = new Sensor_1.Sensor();
        this.hover = new Sensor_1.Sensor();
        this.keyboard = new Sensor_1.Keyboard();
        this.pointer = new Sensor_1.Pointer();
        this.scroll = new Sensor_1.Scroll();
    }
    resetFocus() {
    }
    trackFocus(focus, force) {
        if (focus.length > 0 || force) {
            const f = this.focus;
            f.associatedDataList = switchAssociatedDataList(f.associatedDataList, focus);
            f.revision++;
        }
    }
    doFocusIn(focus) {
        this.trackFocus(focus, false);
    }
    doFocusOut(focus) {
        if (!this.keyboard.down) {
            const f = this.focus;
            reactronic_1.sensitive(reactronic_1.Sensitivity.ReactEvenOnSameValueAssignment, () => switchAssociatedDataList(Sensor_1.EMPTY_ASSOCIATED_DATA_LIST, this.focus.associatedDataList));
            f.revision++;
        }
    }
    doPointerOver(associatedDataList, hoverAssociatedDataList, clientX, clientY) {
        const p = this.pointer;
        const h = this.hover;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
        h.associatedDataList = switchAssociatedDataList(this.hover.associatedDataList, hoverAssociatedDataList);
        h.revision++;
    }
    doPointerLeave(associatedDataList, hoverAssociatedDataList, clientX, clientY) {
    }
    doPointerMove(associatedDataList, pointerId, clientX, clientY) {
        const p = this.pointer;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
    }
    doPointerDown(associatedDataList, focus, pointerId, buttons, clientX, clientY) {
        const p = this.pointer;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
        this.trackFocus(focus, true);
        p.captured = false;
        p.down = buttons;
    }
    doPointerUp(associatedDataList, focus, pointerId, buttons, clientX, clientY) {
        const p = this.pointer;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
        p.up = p.down;
        p.down = Sensor_1.PointerButton.None;
        if (p.captured)
            p.captured = this.releasePointerCapture(pointerId);
    }
    doPointerCancel(associatedDataList, focus, pointerId, buttons, clientX, clientY) {
        const p = this.pointer;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
        p.up = Sensor_1.PointerButton.None;
        p.down = Sensor_1.PointerButton.None;
        p.captured = false;
    }
    doClick(associatedDataList, focus, buttons, clientX, clientY) {
        const p = this.pointer;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
        p.click = buttons;
    }
    doDblClick(associatedDataList, focus, buttons, clientX, clientY) {
        const p = this.pointer;
        Sensors.rememberPointer(p, clientX, clientY);
        p.associatedDataList = associatedDataList;
        p.doubleClick = buttons;
    }
    doTouchStart(associatedDataList, focus) {
        const p = this.pointer;
        p.associatedDataList = associatedDataList;
        this.trackFocus(focus, true);
        p.touched = true;
        p.revision++;
    }
    doTouchEnd(associatedDataList) {
        const p = this.pointer;
        p.associatedDataList = associatedDataList;
        p.touched = false;
        p.revision++;
    }
    doWheel(associatedDataList, focus, deltaX, deltaY, clientX, clientY) {
        const s = this.scroll;
        Sensors.rememberPointer(this.pointer, clientX, clientY);
        s.associatedDataList = associatedDataList;
        s.deltaX = deltaX;
        s.deltaY = deltaY;
        s.revision++;
    }
    doKeyDown(associatedDataList, key) {
        const kb = this.keyboard;
        kb.associatedDataList = associatedDataList;
        kb.up = '';
        reactronic_1.sensitive(reactronic_1.Sensitivity.ReactEvenOnSameValueAssignment, () => kb.down = key);
        kb.modifiers |= Sensors.getKeyAsModifierIfAny(key);
        kb.revision++;
    }
    doKeyUp(associatedDataList, key) {
        const kb = this.keyboard;
        kb.associatedDataList = associatedDataList;
        kb.down = '';
        reactronic_1.sensitive(reactronic_1.Sensitivity.ReactEvenOnSameValueAssignment, () => kb.up = key);
        kb.modifiers &= ~Sensors.getKeyAsModifierIfAny(key);
        kb.revision++;
    }
    setPointerCapture(pointerId) {
        return false;
    }
    releasePointerCapture(pointerId) {
        return false;
    }
    static rememberPointer(p, clientX, clientY) {
        if (p.down === Sensor_1.PointerButton.None) {
            p.associatedDataList = Sensor_1.EMPTY_ASSOCIATED_DATA_LIST;
            p.up = Sensor_1.PointerButton.None;
            p.click = Sensor_1.PointerButton.None;
            p.doubleClick = Sensor_1.PointerButton.None;
        }
        p.previousPositionX = p.positionX;
        p.previousPositionY = p.positionY;
        p.positionX = clientX;
        p.positionY = clientY;
        p.revision++;
    }
    static getKeyAsModifierIfAny(key) {
        let modifier = Sensor_1.KeyboardModifiers.None;
        if (key === 'Control')
            modifier = Sensor_1.KeyboardModifiers.Ctrl;
        else if (key === 'Shift')
            modifier = Sensor_1.KeyboardModifiers.Shift;
        else if (key === 'Alt')
            modifier = Sensor_1.KeyboardModifiers.Alt;
        else if (key === 'Meta')
            modifier = Sensor_1.KeyboardModifiers.Meta;
        return modifier;
    }
    static extractModifierKeys(e) {
        let modifiers = Sensor_1.KeyboardModifiers.None;
        if (e.ctrlKey)
            modifiers |= Sensor_1.KeyboardModifiers.Ctrl;
        else
            modifiers &= ~Sensor_1.KeyboardModifiers.Ctrl;
        if (e.shiftKey)
            modifiers |= Sensor_1.KeyboardModifiers.Shift;
        else
            modifiers &= ~Sensor_1.KeyboardModifiers.Shift;
        if (e.altKey)
            modifiers |= Sensor_1.KeyboardModifiers.Alt;
        else
            modifiers &= ~Sensor_1.KeyboardModifiers.Alt;
        if (e.metaKey)
            modifiers |= Sensor_1.KeyboardModifiers.Meta;
        else
            modifiers &= ~Sensor_1.KeyboardModifiers.Meta;
        return modifiers;
    }
}
exports.Sensors = Sensors;
function grabAssociatedDataList(path, sym, payloadKey, importanceKey, existing) {
    let result = existing;
    let i = 0;
    let j = 0;
    let importance = Number.MIN_SAFE_INTEGER;
    while (i < path.length) {
        const data = path[i][sym];
        if (data !== undefined) {
            const payload = data[payloadKey];
            let imp = data[importanceKey];
            if (payload !== undefined || imp !== undefined) {
                imp = imp !== null && imp !== void 0 ? imp : 0;
                if (imp === importance) {
                    if (result !== existing)
                        payload !== undefined && result.push(payload);
                    else if (payload !== undefined) {
                        if (payload !== existing[j])
                            result = existing.slice(0, j), result.push(payload);
                        else
                            j++;
                    }
                    else {
                        result = existing.slice(0, j);
                    }
                }
                else if (imp > importance) {
                    importance = imp;
                    result = existing;
                    if (payload !== undefined) {
                        if (payload !== existing[0])
                            result = [payload];
                        else
                            j = 1;
                    }
                    else {
                        result = [];
                    }
                }
                else {
                }
            }
        }
        i++;
    }
    if (j === 0 && result === existing && existing.length > 0)
        result = [];
    return result;
}
exports.grabAssociatedDataList = grabAssociatedDataList;
function switchAssociatedDataList(existing, updated) {
    if (updated !== existing) {
        existing.forEach(f => {
            if (f instanceof reactronic_1.ToggleRef && f.value1 !== f.value2)
                f.value = f.value2;
        });
        updated.forEach(x => {
            if (x instanceof reactronic_1.ToggleRef)
                x.value = x.value1;
        });
    }
    return updated;
}
exports.switchAssociatedDataList = switchAssociatedDataList;
