export declare const RefreshParent: void;
export declare type Render<E = unknown, O = void> = (element: E, options: O) => void;
export declare type ComponentRender<O = unknown, E = void> = (render: (options: O) => O, element: E) => void;
export interface MountedInstance<E> {
    native?: E;
    model?: any;
    isResizeSensorEnabled?: boolean;
}
export declare class Manifest<E = unknown, O = void> {
    readonly id: string;
    readonly args: any;
    readonly render: Render<E, O>;
    readonly componentRender: ComponentRender<O, E> | undefined;
    readonly rtti: Rtti<E, O>;
    mounted?: {
        readonly level: number;
        readonly cycle: number;
        instance?: MountedInstance<E> | undefined;
    } | undefined;
    constructor(id: string, args: any, render: Render<E, O>, componentRender: ComponentRender<O, E> | undefined, rtti: Rtti<E, O>, mounted?: {
        readonly level: number;
        readonly cycle: number;
        instance?: MountedInstance<E> | undefined;
    } | undefined);
    annex?: Manifest<E, O>;
    get native(): E | undefined;
}
export interface Rtti<E = unknown, O = void> {
    readonly name: string;
    readonly sorting: boolean;
    render?(m: Manifest<E, O>): void;
    mount?(m: Manifest<E, O>, owner: Manifest, sibling?: Manifest): void;
    reorder?(m: Manifest<E, O>, owner: Manifest, sibling?: Manifest): void;
    unmount?(m: Manifest<E, O>, owner: Manifest, cause: Manifest): void;
}
export declare function manifest<E = unknown, O = void>(id: string, args: any, render: Render<E, O>, componentRender: ComponentRender<O, E> | undefined, rtti: Rtti<E, O>): Manifest<E, O>;
export declare function render(m: Manifest<any, any>): void;
export declare function renderChildrenNow(): void;
export declare function unmount(m: Manifest<any, any>, owner: Manifest, cause: Manifest): void;
export declare function instance<T>(): {
    model?: T;
};
export declare function internalInstance<T>(): MountedInstance<T>;
export declare function cycle(): number;
export declare function trace(enabled: boolean, mask: string, regexp: string): void;
export declare function forAll<E>(action: (e: E) => void): void;
