"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SvgRtti = exports.HtmlRtti = exports.AbstractHtmlRtti = exports.usingParent = void 0;
const reactronic_1 = require("reactronic");
const api_1 = require("../core/api");
function usingParent(e, func, ...args) {
    const outer = AbstractHtmlRtti.current;
    try {
        return func(...args);
    }
    finally {
        AbstractHtmlRtti.current = outer;
    }
}
exports.usingParent = usingParent;
class AbstractHtmlRtti {
    constructor(name, sorting = false) {
        this.name = name;
        this.sorting = sorting;
    }
    render(m) {
        const outer = AbstractHtmlRtti.current;
        try {
            const mounted = m.mounted;
            const instance = mounted.instance;
            const native = instance.native;
            AbstractHtmlRtti.current = native;
            api_1.render(m);
            const data = native.associatedData;
            if (data.drag !== undefined) {
                if (native instanceof HTMLElement && !native.draggable)
                    native.draggable = true;
            }
            else {
                if (native instanceof HTMLElement && native.draggable)
                    native.draggable = false;
            }
            AbstractHtmlRtti.blinkingEffect && blink(native, mounted.cycle);
            if (AbstractHtmlRtti.isDebugAttributeEnabled)
                native.setAttribute('rdbg', `${mounted.cycle}:    ${reactronic_1.Reactronic.why()}`);
        }
        finally {
            AbstractHtmlRtti.current = outer;
        }
    }
    mount(m, owner, sibling) {
        var _a, _b, _c, _d, _e;
        const parent = (_c = (_b = (_a = owner.mounted) === null || _a === void 0 ? void 0 : _a.instance) === null || _b === void 0 ? void 0 : _b.native) !== null && _c !== void 0 ? _c : AbstractHtmlRtti.current;
        const native = this.createElement(m);
        native.id = m.id;
        if (!owner.rtti.sorting) {
            if (sibling !== undefined) {
                const prev = (_e = (_d = sibling.mounted) === null || _d === void 0 ? void 0 : _d.instance) === null || _e === void 0 ? void 0 : _e.native;
                if (prev instanceof Element)
                    parent.insertBefore(native, prev.nextSibling);
            }
            else
                parent.insertBefore(native, parent.firstChild);
        }
        else
            parent.appendChild(native);
        m.mounted.instance.native = native;
    }
    reorder(m, owner, sibling) {
        var _a, _b, _c, _d, _e, _f, _g;
        const parent = (_c = (_b = (_a = owner.mounted) === null || _a === void 0 ? void 0 : _a.instance) === null || _b === void 0 ? void 0 : _b.native) !== null && _c !== void 0 ? _c : AbstractHtmlRtti.current;
        const prev = (_e = (_d = sibling === null || sibling === void 0 ? void 0 : sibling.mounted) === null || _d === void 0 ? void 0 : _d.instance) === null || _e === void 0 ? void 0 : _e.native;
        const native = (_g = (_f = m.mounted) === null || _f === void 0 ? void 0 : _f.instance) === null || _g === void 0 ? void 0 : _g.native;
        if (native && prev instanceof Element && prev.nextSibling !== native) {
            parent.insertBefore(native, prev.nextSibling);
        }
    }
    unmount(m, owner, cause) {
        var _a, _b;
        const native = (_b = (_a = m.mounted) === null || _a === void 0 ? void 0 : _a.instance) === null || _b === void 0 ? void 0 : _b.native;
        if (!AbstractHtmlRtti.unmounting && native && native.parentElement) {
            AbstractHtmlRtti.unmounting = native;
            try {
                native.remove();
                api_1.unmount(m, owner, cause);
            }
            finally {
                AbstractHtmlRtti.unmounting = undefined;
            }
        }
        else {
            api_1.unmount(m, owner, cause);
        }
    }
    static set blinkingEffect(value) {
        if (value === undefined) {
            api_1.forAll((e) => {
                if (e instanceof HTMLElement)
                    e.classList.remove(`${AbstractHtmlRtti.blinkingEffect}-0`, `${AbstractHtmlRtti.blinkingEffect}-1`);
            });
        }
        AbstractHtmlRtti._blinkingEffect = value;
    }
    static get blinkingEffect() {
        return AbstractHtmlRtti._blinkingEffect;
    }
}
exports.AbstractHtmlRtti = AbstractHtmlRtti;
AbstractHtmlRtti.isDebugAttributeEnabled = false;
AbstractHtmlRtti.current = (_a = global.document) === null || _a === void 0 ? void 0 : _a.body;
AbstractHtmlRtti.unmounting = undefined;
AbstractHtmlRtti._blinkingEffect = undefined;
function blink(e, cycle) {
    const n1 = cycle % 2;
    const n2 = 1 >> n1;
    e.classList.toggle(`${AbstractHtmlRtti.blinkingEffect}-${n1}`, true);
    e.classList.toggle(`${AbstractHtmlRtti.blinkingEffect}-${n2}`, false);
}
class HtmlRtti extends AbstractHtmlRtti {
    createElement(m) {
        return document.createElement(m.rtti.name);
    }
}
exports.HtmlRtti = HtmlRtti;
class SvgRtti extends AbstractHtmlRtti {
    createElement(e) {
        return document.createElementNS('http://www.w3.org/2000/svg', e.rtti.name);
    }
}
exports.SvgRtti = SvgRtti;
