"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HtmlSensors = void 0;
const reactronic_1 = require("reactronic");
const api_1 = require("../core/api");
const System_1 = require("../core/System");
const HtmlApiExt_1 = require("./HtmlApiExt");
const HtmlSensor_1 = require("./HtmlSensor");
class HtmlSensors extends api_1.Sensors {
    constructor() {
        super();
        this.currentEvent = undefined;
        this.eventSource = undefined;
        this.drag = new HtmlSensor_1.HtmlDrag(reactronic_1.Ref.to(this).currentEvent);
        this.resize = new HtmlSensor_1.HtmlResize();
        this.resizeObserver = new ResizeObserver(this.onResize);
    }
    observeResizeOfRenderingElement(value) {
        const instance = System_1.internalInstance();
        if (value !== instance.isResizeSensorEnabled) {
            if (value)
                this.resizeObserver.observe(instance.native);
            else
                this.resizeObserver.unobserve(instance.native);
            instance.isResizeSensorEnabled = value;
        }
    }
    preventDefault() {
        if (this.currentEvent !== undefined)
            this.currentEvent.preventDefault();
    }
    stopPropagation() {
        if (this.currentEvent !== undefined)
            this.currentEvent.stopPropagation();
    }
    listen(element, enabled = true) {
        const existing = this.eventSource;
        if (element !== existing) {
            if (existing) {
                existing.removeEventListener('focusin', this.onFocusIn, { capture: true });
                existing.removeEventListener('focusout', this.onFocusOut, { capture: true });
                existing.removeEventListener('pointermove', this.onPointerMove, { capture: true });
                existing.removeEventListener('pointerdown', this.onPointerDown, { capture: true });
                existing.removeEventListener('pointerup', this.onPointerUp, { capture: true });
                existing.removeEventListener('lostpointercapture', this.onLostPointerCapture, { capture: true });
                existing.removeEventListener('click', this.onClick, { capture: true });
                existing.removeEventListener('dblclick', this.onDblClick, { capture: true });
                existing.removeEventListener('touchstart', this.onTouchStart, { capture: true });
                existing.removeEventListener('touchend', this.onTouchEnd, { capture: true });
                existing.removeEventListener('wheel', this.onWheel, { capture: true });
                existing.removeEventListener('keydown', this.onKeyDown, { capture: true });
                existing.removeEventListener('keyup', this.onKeyUp, { capture: true });
                existing.removeEventListener('dragstart', this.onDragStart, { capture: true });
                existing.removeEventListener('dragover', this.onDragOver, { capture: true });
                existing.removeEventListener('drop', this.onDrop, { capture: true });
                existing.removeEventListener('dragend', this.onDragEnd, { capture: true });
            }
            this.eventSource = element;
            if (element && enabled) {
                element.addEventListener('focusin', this.onFocusIn, { capture: true });
                element.addEventListener('focusout', this.onFocusOut, { capture: true });
                element.addEventListener('pointerover', this.onPointerOver, { capture: true });
                element.addEventListener('pointermove', this.onPointerMove, { capture: true });
                element.addEventListener('pointerdown', this.onPointerDown, { capture: true });
                element.addEventListener('pointerup', this.onPointerUp, { capture: true });
                element.addEventListener('lostpointercapture', this.onLostPointerCapture, { capture: true });
                element.addEventListener('click', this.onClick, { capture: true });
                element.addEventListener('dblclick', this.onDblClick, { capture: true });
                element.addEventListener('touchstart', this.onTouchStart, { capture: true });
                element.addEventListener('touchend', this.onTouchEnd, { capture: true });
                element.addEventListener('wheel', this.onWheel, { capture: true });
                element.addEventListener('keydown', this.onKeyDown, { capture: true });
                element.addEventListener('keyup', this.onKeyUp, { capture: true });
                element.addEventListener('dragstart', this.onDragStart, { capture: true });
                element.addEventListener('dragover', this.onDragOver, { capture: true });
                element.addEventListener('drop', this.onDrop, { capture: true });
                element.addEventListener('dragend', this.onDragEnd, { capture: true });
            }
        }
    }
    resetFocus() {
        var _a, _b, _c;
        const ei = (_b = (_a = this.eventSource) === null || _a === void 0 ? void 0 : _a.associatedData) === null || _b === void 0 ? void 0 : _b.focus;
        this.trackFocus(ei ? [ei] : [], true);
        (_c = this.eventSource) === null || _c === void 0 ? void 0 : _c.focus();
    }
    onFocusIn(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doFocusIn(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList));
    }
    onFocusOut(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doFocusOut(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList));
    }
    onPointerOver(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doPointerOver(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'hover', 'hoverImportance', this.hover.associatedDataList), e.clientX, e.clientY);
    }
    onPointerMove(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doPointerMove(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), e.pointerId, e.clientX, e.clientY);
    }
    onPointerDown(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doPointerDown(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList), e.pointerId, e.buttons, e.clientX, e.clientY);
    }
    onPointerUp(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doPointerUp(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList), e.pointerId, e.buttons, e.clientX, e.clientY);
    }
    onLostPointerCapture(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doPointerCancel(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList), e.pointerId, e.buttons, e.clientX, e.clientY);
    }
    onClick(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doClick(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList), api_1.PointerButton.Left, e.clientX, e.clientY);
    }
    onDblClick(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doDblClick(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList), api_1.PointerButton.Left, e.clientX, e.clientY);
    }
    onTouchStart(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doTouchStart(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList));
    }
    onTouchEnd(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doTouchEnd(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'pointer', 'pointerImportance', this.pointer.associatedDataList));
    }
    onWheel(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doWheel(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'scroll', 'scrollImportance', this.scroll.associatedDataList), api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'focus', 'focusImportance', this.focus.associatedDataList), e.deltaX, e.deltaY, e.clientX, e.clientY);
    }
    onKeyDown(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doKeyDown(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'keyboard', 'keyboardImportance', this.keyboard.associatedDataList), e.key);
    }
    onKeyUp(e) {
        const path = e.composedPath();
        this.currentEvent = e;
        this.doKeyUp(api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'keyboard', 'keyboardImportance', this.keyboard.associatedDataList), e.key);
    }
    onDragStart(e) {
        const path = e.composedPath();
        const associatedDataList = api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'drag', 'dragImportance', this.drag.associatedDataList);
        const d = this.drag;
        this.currentEvent = e;
        d.associatedDataList = associatedDataList;
        d.stage = HtmlSensor_1.DragStage.Started;
        d.draggingObject = associatedDataList[0];
        d.draggingStartAtX = e.clientX;
        d.draggingStartAtY = e.clientY;
        d.draggingModifiers = this.keyboard.modifiers;
        d.revision++;
    }
    onDragOver(e) {
        const path = e.composedPath();
        const d = this.drag;
        const p = this.pointer;
        this.currentEvent = e;
        api_1.Sensors.rememberPointer(p, e.clientX, e.clientY);
        d.associatedDataList = api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'drag', 'dragImportance', this.drag.associatedDataList);
        reactronic_1.sensitive(reactronic_1.Sensitivity.ReactEvenOnSameValueAssignment, () => d.stage = HtmlSensor_1.DragStage.Dragging);
        d.revision++;
    }
    onDrop(e) {
        const path = e.composedPath();
        const d = this.drag;
        this.currentEvent = e;
        d.associatedDataList = api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'drag', 'dragImportance', this.drag.associatedDataList);
        d.droppedAtX = e.clientX;
        d.droppedAtY = e.clientY;
        d.stage = HtmlSensor_1.DragStage.Dropped;
        d.revision++;
    }
    onDragEnd(e) {
        const path = e.composedPath();
        const d = this.drag;
        this.currentEvent = e;
        d.associatedDataList = api_1.grabAssociatedDataList(path, HtmlApiExt_1.SymAssociatedData, 'drag', 'dragImportance', this.drag.associatedDataList);
        d.stage = HtmlSensor_1.DragStage.Finished;
        d.draggingStartAtX = Infinity;
        d.draggingStartAtY = Infinity;
        d.draggingModifiers = api_1.KeyboardModifiers.None;
        d.draggingObject = undefined;
        d.droppedAtX = e.clientX;
        d.droppedAtY = e.clientY;
        d.revision++;
    }
    onResize(entries) {
        const r = this.resize;
        r.revision++;
        r.resizedElements = entries.map(entry => {
            const element = entry.target;
            return {
                borderBoxSize: entry.borderBoxSize,
                contentBoxSize: entry.contentBoxSize,
                contentRect: entry.contentRect,
                resizeData: element.associatedData.resize,
            };
        });
        r.associatedDataList = entries.map(x => {
            const element = x.target;
            return element.associatedData;
        });
    }
    setPointerCapture(pointerId) {
        var _a;
        (_a = this.eventSource) === null || _a === void 0 ? void 0 : _a.setPointerCapture(pointerId);
        return this.eventSource instanceof HTMLElement;
    }
    releasePointerCapture(pointerId) {
        var _a;
        (_a = this.eventSource) === null || _a === void 0 ? void 0 : _a.releasePointerCapture(pointerId);
        return false;
    }
}
__decorate([
    reactronic_1.unobservable,
    __metadata("design:type", ResizeObserver)
], HtmlSensors.prototype, "resizeObserver", void 0);
__decorate([
    reactronic_1.unobservable,
    __metadata("design:type", Object)
], HtmlSensors.prototype, "currentEvent", void 0);
__decorate([
    reactronic_1.transaction,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "listen", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "resetFocus", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [FocusEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onFocusIn", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [FocusEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onFocusOut", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onPointerOver", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onPointerMove", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onPointerDown", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onPointerUp", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onLostPointerCapture", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MouseEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onClick", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MouseEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onDblClick", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Event]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onTouchStart", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Event]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onTouchEnd", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [WheelEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onWheel", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [KeyboardEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onKeyDown", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [KeyboardEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onKeyUp", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [DragEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onDragStart", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [DragEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onDragOver", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [DragEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onDrop", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [DragEvent]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onDragEnd", null);
__decorate([
    reactronic_1.transaction,
    reactronic_1.trace(reactronic_1.TraceLevel.Suppress),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], HtmlSensors.prototype, "onResize", null);
exports.HtmlSensors = HtmlSensors;
